/**
 * Copyright (C), 2021-2021, 夸克科技有限公司
 * FileName: androidNativeMethods
 * Author: Administrator
 * Date: 2021/10/26 17:16
 * Description: 安卓原生方法
 * History:
 * <author> <time> <version> <desc>
 * 作者姓名 修改时间 版本号 描述
 */
package quake.quake_android_native_methods;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.widget.Toast;

import com.huantansheng.easyphotos.EasyPhotos;
import com.huantansheng.easyphotos.models.album.entity.Photo;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;

import quake.quake_android_native_methods.ui.GetPosition;
import quake.quake_android_native_methods.ui.Scanning;
import quake.quake_android_native_methods.ui.filePage.FilePage;
import quake.quake_android_native_methods.utils.BitmapUtils;
import quake.quake_android_native_methods.utils.Constant;
import quake.quake_android_native_methods.utils.FileUtils;
import quake.quake_android_native_methods.utils.ImageUtils;
import quake.quake_android_native_methods.utils.JSONUtils.JsonUtils;
import quake.quake_android_native_methods.utils.UriUtils;
import quake.quake_android_native_methods.utils.VideoUtils.VideoUtils;

/**
 * @ClassName: androidNativeMethods
 * @Description: java类作用描述
 * 安卓原生方法
 * @Author: Administrator
 * @Date: 2021/10/26 17:16
 */
public class AndroidNativeMethods {
    private static final String TAG = "androidNativeMethods";


    /**
     * 选择文件
     *
     * @param activity
     */
    public static void chooseFile(Activity activity) {
        Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        //设置doc,docx,ppt,pptx,pdf 5种类型
        intent.setType("application/msword|application/vnd.openxmlformats-officedocument.wordprocessingml.document" +
                "|application/vnd.ms-powerpoint|application/vnd.openxmlformats-officedocument.presentationml.presentation|application/pdf" +
                "|application/vnd.ms-excel|application/x-excel");
        //在API>=19之后设置多个类型采用以下方式，setType不再支持多个类型
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
//            intent.putExtra(Intent.EXTRA_MIME_TYPES,
//                    Constants.mimeTypes);
//        }
        activity.startActivityForResult(intent, Constant.REQUEST_CHOOSE_FILE);
    }

    public static void browseFile(Context context, Activity activity, String data) {
        Log.i("AndroidToJs", "openFile result=" + data);
        Intent intent = new Intent(context, FilePage.class);
        intent.putExtra("url", data);
        activity.startActivityForResult(intent,Constant.REQUEST_BROWSE_FILE);

    }

    public static void choosePicture(Activity activity) {
        EasyPhotos.preLoad(activity);
        EasyPhotos.createAlbum(activity, false, true
                , GlideEngine.getInstance())
                .setCount(9).start(Constant.REQUEST_IMAGE);
    }

    public static void chooseVideo(Activity activity) {
        Intent intent = new Intent();
        intent.setAction(Intent.ACTION_PICK);
        intent.setType("video/*");
        activity.startActivityForResult(intent, Constant.REQUEST_VIDEO);
    }

    public static void videoRecording(Activity activity) {
        Log.d(TAG, "onClick: 点了摄像");
                    /*        File file = new File(Environment.getExternalStorageDirectory().getPath() + "/quake_video/"+System.currentTimeMillis()+".mp4");//设置录像存储路径
//        Uri uri = Uri.fromFile(file);   // 将路径转换为Uri对象
        Uri uri = FileProvider.getUriForFile(mContext, mContext.getApplicationContext().getPackageName() + ".provider", file);
        Log.i("AndroidToJs", "chooseVideo uri=" + uri);*/
        Intent intent = new Intent(MediaStore.ACTION_VIDEO_CAPTURE);
        intent.putExtra(MediaStore.EXTRA_DURATION_LIMIT, 2000);
        intent.putExtra(MediaStore.EXTRA_VIDEO_QUALITY, 1);
        intent.putExtra(MediaStore.EXTRA_FINISH_ON_COMPLETION, true);
//        intent.putExtra(MediaStore.EXTRA_OUTPUT, uri);
        activity.startActivityForResult(intent, Constant.REQUEST_TAKE_VIDEO);
    }

    public static void photograph(Context context, Activity activity) {
        Log.d(TAG, "onClick: 点了拍照");
        Uri imageUri = UriUtils.createImageUri(context);
//                File file = new File(Environment.getExternalStorageDirectory(), "quake_");
//                Uri imageUri = Uri.fromFile(file);
        Intent intent = new Intent();
        intent.setAction(MediaStore.ACTION_IMAGE_CAPTURE);
        intent.putExtra(MediaStore.EXTRA_OUTPUT, imageUri);//如果不设置EXTRA_OUTPUT getData()  获取的是bitmap数据  是压缩后的
        Log.d(TAG, "onClick: imageURI:" + imageUri);
        String imgpath = ImageUtils.getImageAbsolutePath(context, imageUri);
        Log.d(TAG, "onClick: imgpath:" + imgpath);
        activity.startActivityForResult(intent, Constant.TAKE_PHOTO_REQUEST);
    }

    public static void soundRecording() {

    }

    public static void sendLocation(Context context, Activity activity) {
        Log.i("AndroidToJs", "choosePosition result=");
        Intent intent = new Intent(context, GetPosition.class);
        activity.startActivityForResult(intent, Constant.REQUEST_POSITION);
    }

    public static void scanQRCode(Context context, Activity activity) {
        Log.i("AndroidToJs", "scanQRCode result=");
        Intent intent = new Intent(context, Scanning.class);
        activity.startActivityForResult(intent, Constant.SCAN_QR_CODE);
    }


    /**
     * 获取拍照信息
     *
     * @param context 上下文
     * @return
     */
    public static JSONObject getPhotoInfo(Context context) {
        SharedPreferences sharedPreferences = context.getSharedPreferences("photoPath", Activity.MODE_PRIVATE);
        String photoPath = sharedPreferences.getString("path", "");
        Bitmap imgThumbnail = BitmapUtils.getBitmapFromUri(photoPath);
        int photoWidth = imgThumbnail.getWidth();
        int photoHeight = imgThumbnail.getHeight();
        String photoName = FileUtils.getFileNameWithSuffix(photoPath);
        JSONObject jsonObject = new JSONObject();
        jsonObject = JsonUtils.inputValue2JSONObject(jsonObject, "photoPath", photoPath,
                "photoWidth", photoWidth, "photoHeight", photoHeight,
                "photoName", photoName);
        return jsonObject;
    }

    /**
     * 获取相册选中的图片的所有信息
     *
     * @param data    选中的图片数据
     * @param context 上下文
     * @return 图片信息保存在JSONArray中
     */
    public static JSONArray getPhotoListInfo(Intent data, Context context) {
        ArrayList<Photo> resultPhotos = data.getParcelableArrayListExtra(EasyPhotos.RESULT_PHOTOS);
        Uri imageUri;
        String url;
        int imageHeight;
        int imageWidth;
        String imageName;
        JSONArray jsonArray = new JSONArray();
        for (int i = 0; i < resultPhotos.size(); i++) {
            JSONObject jsonObject = new JSONObject();
            Photo photo = resultPhotos.get(i);
            imageUri = photo.uri;
            Log.d(TAG, "getMultiplePicturesBean: uri转path ：" + UriUtils.getPathFromUri(context, imageUri));
            url = photo.path;
            imageHeight = photo.height;
            imageWidth = photo.width;
            imageName = photo.name;
            jsonObject = JsonUtils.inputValue2JSONObject(jsonObject,
                    "picName", imageName, "url", url, "w", imageWidth,
                    "h", imageHeight, "imageUri", imageUri);
            jsonArray.put(jsonObject);
        }
        return jsonArray;
    }

    /**
     * 获取视频信息
     *
     * @param data    视频数据
     * @param context 上下文
     * @return
     */
    public static JSONObject getVideoInfo(Intent data, Context context) {
        Uri uri = data.getData();
        Toast.makeText(context, "摄像", Toast.LENGTH_SHORT).show();
        //获取视频本地路径
        String videoUrl = VideoUtils.getAllVideoOnDevice(context, uri);
        String videoName = FileUtils.getFileNameWithSuffix(videoUrl);
        //获取视频时长
        int videoDuration = VideoUtils.getVideoDuration(context, uri);
        String file = FileUtils.getDiskCacheDir(context);
        String coverName = "videoCover" + System.currentTimeMillis() + ".jpeg";
        File appCacheDir = new File(file, coverName);
        //封面本地地址
        String coverUrl = appCacheDir.getPath();
        //获取视频的第一帧作为封面
        Bitmap bitmap = UriUtils.getVideoFirstFrameByUri(context, uri);
        int coverWidth = bitmap.getWidth();
        int coverHeight = bitmap.getHeight();
        BitmapUtils.saveBitmap(coverUrl, bitmap);
        JSONObject jsonObject = new JSONObject();
        JsonUtils.inputValue2JSONObject(jsonObject, "videoName", videoName,
                "videoUrl", videoUrl, "videoDuration", videoDuration,
                "coverName", coverName, "coverUrl", coverUrl,
                "coverWidth", coverWidth, "coverHeight", coverHeight);
        return jsonObject;
    }

    /**
     * 获取文件信息
     *
     * @param data    文件数据
     * @param context 上下文
     * @return
     */
    public static JSONObject getFileInfo(Intent data, Context context) {
        Uri uri = data.getData();
        int fileSize = FileUtils.getFileSizeByUri(context, uri);
        String fileUrl = ImageUtils.getImageAbsolutePath(context, uri);
        String fileName = FileUtils.getFileNameWithSuffix(fileUrl);
        JSONObject jsonObject = new JSONObject();
        jsonObject = JsonUtils.inputValue2JSONObject(jsonObject, "fileName", fileName,
                "fileSize", fileSize, "fileUrl", fileUrl);
        return jsonObject;
    }

    /**
     * 获取定位信息
     *
     * @param data 定位数据
     * @return
     */
    public static JSONObject getLocationInfo(Intent data) {
        Bundle bundle = data.getExtras();
        String locScreenShotUrl = bundle.getString("picPath");
        String locScreenShotName = bundle.getString("picName");
        String location = bundle.getString("position");
        Double latitude = bundle.getDouble("latitude");
        Log.d(TAG, "getLocationInfo: latitude = "+latitude);
        Double longitude = bundle.getDouble("longitude");
        JSONObject jsonObject = new JSONObject();
        jsonObject = JsonUtils.inputValue2JSONObject(jsonObject, "locScreenShotUrl", locScreenShotUrl,
                "locScreenShotName", locScreenShotName, "location", location,
                "latitude", latitude, "longitude", longitude);
        return jsonObject;
    }

    public static JSONObject getScanInfo(Intent data) {
        JSONObject jsonObject = new JSONObject();
        String result = data.getStringExtra("result");
        int code = data.getIntExtra("code",1400);
        jsonObject = JsonUtils.inputValue2JSONObject(jsonObject, "result",result,
                "code",code);
        return jsonObject;
    }

}
