package quake.quake_android_native_methods;

import android.app.Activity;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;

import com.huantansheng.easyphotos.EasyPhotos;
import com.huantansheng.easyphotos.models.album.entity.Photo;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import quake.quake_android_native_methods.utils.BitmapUtils;
import quake.quake_android_native_methods.utils.Constant;
import quake.quake_android_native_methods.utils.FileUtils;
import quake.quake_android_native_methods.utils.JSONUtils.JsonUtils;
import quake.quake_android_native_methods.utils.UriUtils;
import quake.quake_android_native_methods.utils.VideoUtils.VideoUtils;

import static android.app.Activity.RESULT_CANCELED;
import static android.app.Activity.RESULT_OK;

/**
 * QuakeAndroidNativeMethodsPlugin
 */
public class QuakeAndroidNativeMethodsPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware,
        PluginRegistry.ActivityResultListener {
    private static final String TAG = "NativeMethodsPlugin";
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    static String CHANNEL_NAME = "quake_android_native_methods";
    private MethodChannel channel;
    private Context mContext;
    private Activity mActivity;
    Result mResult;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), CHANNEL_NAME);
        channel.setMethodCallHandler(this);
        mContext = flutterPluginBinding.getApplicationContext();
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        mResult = result;
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (call.method.equals("chooseImgFromGallery")) {
            AndroidNativeMethods.choosePicture(mActivity);
        } else if (call.method.equals("photograph")) {
            AndroidNativeMethods.photograph(mContext, mActivity);
        } else if (call.method.equals("chooseVideoFromSelector")) {
            AndroidNativeMethods.chooseVideo(mActivity);
        } else if (call.method.equals("videoRecording")) {
            AndroidNativeMethods.videoRecording(mActivity);
        } else if (call.method.equals("chooseFileFromSelector")) {
            AndroidNativeMethods.chooseFile(mActivity);
        } else if (call.method.equals("browseFile")) {
            String url = call.argument("url");
            Log.d(TAG, "onMethodCall: 文件Url=" + url);
            AndroidNativeMethods.browseFile(mContext, mActivity, url);
        } else if (call.method.equals("soundRecording")) {

        } else if (call.method.equals("sendLocation")) {
            AndroidNativeMethods.sendLocation(mContext, mActivity);
        } else if (call.method.equals("scanQRCode")) {
            AndroidNativeMethods.scanQRCode(mContext, mActivity);
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        mActivity = binding.getActivity();
        binding.addActivityResultListener(this);
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
        onAttachedToActivity(binding);
    }

    @Override
    public void onDetachedFromActivity() {

    }

    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
            //拍照
            case Constant.TAKE_PHOTO_REQUEST: {
                if (resultCode == RESULT_OK) {
                    Toast.makeText(mContext, "拍照", Toast.LENGTH_SHORT).show();
                    JSONObject jsonObject = AndroidNativeMethods.getPhotoInfo(mContext);
                    mResult.success(jsonObject.toString());
                }
            }
            break;
            //相册
            case Constant.REQUEST_IMAGE:
                Toast.makeText(mContext, "选择图片", Toast.LENGTH_SHORT).show();
                if (resultCode == RESULT_OK) {
                    JSONArray jsonArray = AndroidNativeMethods.getPhotoListInfo(data, mContext);
                    mResult.success(jsonArray.toString());
                }
                break;
            //摄像
            case Constant.REQUEST_TAKE_VIDEO:
                if (resultCode == RESULT_OK) {
                    if (data != null) {
                        JSONObject jsonObject = AndroidNativeMethods.getVideoInfo(data, mContext);
                        mResult.success(jsonObject.toString());
                    }
                } else if (resultCode == RESULT_CANCELED) {
                    Toast.makeText(mContext, "取消了摄像", Toast.LENGTH_SHORT).show();
                }
                break;
            //视频
            case Constant.REQUEST_VIDEO:
                if (data != null) {
                    Toast.makeText(mContext, "选择视频", Toast.LENGTH_SHORT).show();
                    JSONObject jsonObject = AndroidNativeMethods.getVideoInfo(data, mContext);
                    mResult.success(jsonObject.toString());
                }
                break;
            //文件
            case Constant.REQUEST_CHOOSE_FILE:
                if (data != null) {
                    Toast.makeText(mContext, "选择文件", Toast.LENGTH_SHORT).show();
                    JSONObject jsonObject = AndroidNativeMethods.getFileInfo(data, mContext);
                    mResult.success(jsonObject.toString());
                }
                break;
            //文件
            case Constant.REQUEST_BROWSE_FILE:
                Log.d(TAG, "onActivityResult: 浏览文件");
                if (data!= null) {
                    Toast.makeText(mContext, "浏览文件", Toast.LENGTH_SHORT).show();
                    String snapShotUrl = data.getStringExtra("snapShotUrl");
//                    JSONObject jsonObject = AndroidNativeMethods.getFileInfo(data, mContext);
                    JSONObject url = new JSONObject();
                    url = JsonUtils.inputValue2JSONObject(url,"snapShotUrl",snapShotUrl);
                    mResult.success(url.toString());
                }
                break;
            //定位
            case Constant.REQUEST_POSITION:
                if (data != null) {
                    Toast.makeText(mContext, "定位", Toast.LENGTH_SHORT).show();
                    JSONObject jsonObject = AndroidNativeMethods.getLocationInfo(data);
                    mResult.success(jsonObject.toString());
                }
                break;
            case Constant.SCAN_QR_CODE:
                if(data!=null){
                    JSONObject result = AndroidNativeMethods.getScanInfo(data);
                    mResult.success(result.toString());
                }
                break;
        }
        return true;
    }
}
