package quake.quake_android_native_methods.ui;

import android.content.Intent;
import android.graphics.Bitmap;
import android.location.Location;
import android.os.Build;
import android.os.Bundle;
import android.os.PersistableBundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import com.amap.api.maps2d.AMap;
import com.amap.api.maps2d.CameraUpdateFactory;
import com.amap.api.maps2d.MapView;
import com.amap.api.maps2d.UiSettings;
import com.amap.api.maps2d.model.BitmapDescriptorFactory;
import com.amap.api.maps2d.model.LatLng;
import com.amap.api.maps2d.model.LatLngBounds;
import com.amap.api.maps2d.model.Marker;
import com.amap.api.maps2d.model.MarkerOptions;
import com.amap.api.maps2d.model.MyLocationStyle;
import com.amap.api.services.core.LatLonPoint;
import com.amap.api.services.geocoder.GeocodeResult;
import com.amap.api.services.geocoder.GeocodeSearch;
import com.amap.api.services.geocoder.RegeocodeQuery;
import com.amap.api.services.geocoder.RegeocodeResult;



import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import quake.quake_android_native_methods.R;
import quake.quake_android_native_methods.baseMvp.BasePresenter;
import quake.quake_android_native_methods.databinding.GetPositionBinding;
import quake.quake_android_native_methods.ui.base.SlidingBackActivity;
import quake.quake_android_native_methods.utils.BitmapUtils;
import quake.quake_android_native_methods.utils.DatabaseUtils;
import quake.quake_android_native_methods.utils.FileUtils;
import quake.quake_android_native_methods.utils.KeyUtils;

public class GetPosition extends SlidingBackActivity implements AMap.OnMyLocationChangeListener,
        GeocodeSearch.OnGeocodeSearchListener, View.OnClickListener, TextWatcher,
        AMap.InfoWindowAdapter, AMap.OnMarkerClickListener, AMap.OnMapClickListener,
        AMap.OnMapScreenShotListener {
    private MapView mMapView;
    private AMap aMap;
    private RegeocodeQuery query;
    private GeocodeSearch geocodeSearch;
    private String TAG = "GetPosition";
    private String address;
    private Marker clickMarker;

    private EditText editPosition;
    private LinearLayout linearLayout;
    private Button send;
    private Button backward;
    private boolean isFirst = true;
    private double latitude;
    private double longitude;
    private int REQUEST_POSITION = 6;
    private boolean justShowPosition = false;   //判断是发送定位还是查看定位
    private LatLngBounds.Builder builder;
    private Intent intent;

    private String action;

    private GetPositionBinding binding;

    @RequiresApi(api = Build.VERSION_CODES.N)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        getPositionIntent();
        System.out.println(KeyUtils.sHA1(this));
        initViews();
        initMap();
        mMapView.onCreate(savedInstanceState);
        String key = KeyUtils.sHA1(this);
        Log.d(TAG, "onCreate: 鉴权="+key);
    }

    @Override
    protected BasePresenter createPresenter() {
        return null;
    }

    //获取定位信息
    private void getPositionIntent() {
        intent = getIntent();
        Log.d(TAG, "getPositionIntent: intent=" + intent.getExtras());
        if (intent.getExtras() != null) {
            //仅仅只是查看定位
            justShowPosition = true;
            longitude = DatabaseUtils.convertToDouble(intent.getStringExtra("longitude"), 0);
            latitude = DatabaseUtils.convertToDouble(intent.getStringExtra("latitude"), 0);
            address = intent.getStringExtra("address");
            Log.d(TAG, "getPositionIntent: longitude=" + longitude + ",latitude=" + latitude + ",address=" + address);
        }
    }

    //初始化地图
    private void initMap() {
        mMapView = binding.map;
        if (aMap == null) {
            aMap = mMapView.getMap();
        }
        MyLocationStyle myLocationStyle = new MyLocationStyle();//初始化定位蓝点样式类myLocationStyle.myLocationType(MyLocationStyle.LOCATION_TYPE_LOCATION_ROTATE);
        // 连续定位、且将视角移动到地图中心点，定位点依照设备方向旋转，并且会跟随设备移动。（1秒1次定位）如果不设置myLocationType，默认也会执行此种模式。
        if (!justShowPosition) {
            myLocationStyle.myLocationType(MyLocationStyle.LOCATION_TYPE_LOCATE);
            myLocationStyle.interval(1000); //设置连续定位模式下的定位间隔，只在连续定位模式下生效，单次定位模式下不会生效。单位为毫秒。
            myLocationStyle.myLocationIcon(BitmapDescriptorFactory.fromResource(R.mipmap.location));
//        myLocationStyle.strokeWidth(0);
            myLocationStyle.strokeColor(0x00000000);
            myLocationStyle.radiusFillColor(0x00000000);
//        myLocationStyle.myLocationIcon(BitmapDescriptorFactory.fromBitmap(BitmapFactory.decodeResource(getResources(),R.mipmap.location)));
            //设置定位蓝点的icon图标方法，需要用到BitmapDescriptor类对象作为参数。
            aMap.setOnMyLocationChangeListener(this);
            aMap.setMyLocationStyle(myLocationStyle);//设置定位蓝点的Style
            //设置默认定位按钮是否显示，非必需设置。
            aMap.setMyLocationEnabled(true);// 设置为true表示启动显示定位蓝点，false表示隐藏定位蓝点并不进行定位，默认是false。
        } else {
            LatLng latLng = new LatLng(latitude, longitude);
            drawMarker(latLng, address);
            aMap.animateCamera(CameraUpdateFactory.newLatLngBounds(builder.build(), 0));
            aMap.setInfoWindowAdapter(this);
            aMap.setOnMarkerClickListener(this);
            aMap.setOnMapClickListener(this);
        }
        UiSettings mUiSettings;
        mUiSettings = aMap.getUiSettings();
        mUiSettings.setZoomControlsEnabled(false);
        geocodeSearch = new GeocodeSearch(this);
        geocodeSearch.setOnGeocodeSearchListener(this);

    }


    //绘制标记点
    private void drawMarker(LatLng latLng, String location) {
        builder = new LatLngBounds.Builder();
        MarkerOptions markerOptions = new MarkerOptions();
        markerOptions.position(latLng).title(location);
        Marker marker = aMap.addMarker(markerOptions);

        builder.include(latLng);
    }

    @Override
    public void onSaveInstanceState(@NonNull Bundle outState, @NonNull PersistableBundle outPersistentState) {
        super.onSaveInstanceState(outState, outPersistentState);
        //在activity执行onSaveInstanceState时执行mMapView.onSaveInstanceState (outState)，保存地图当前的状态
        mMapView.onSaveInstanceState(outState);
    }

    //定位改变监听
    @Override
    public void onMyLocationChange(Location location) {
        latitude = location.getLatitude();
        longitude = location.getLongitude();
        LatLonPoint point = new LatLonPoint(latitude, longitude);
        query = new RegeocodeQuery(point, 200, GeocodeSearch.AMAP);
        geocodeSearch.getFromLocationAsyn(query);
        Log.d("location_handler", "纬度: " + latitude + ",经度: " + longitude);
    }

    //获取地址描述数据
    @Override
    public void onRegeocodeSearched(RegeocodeResult regeocodeResult, int i) {
        String formatAddress = regeocodeResult.getRegeocodeAddress().getFormatAddress();
        String street = regeocodeResult.getRegeocodeAddress().getStreetNumber().getStreet();
        String streetNum = regeocodeResult.getRegeocodeAddress().getStreetNumber().getNumber();
        address = formatAddress + "(" + street + streetNum + ")";
        Log.d(TAG, "onRegeocodeSearched: address=" + address);
        if (isFirst) {
            editPosition.setText(address);
            isFirst = false;
            LatLngBounds.Builder builder = new LatLngBounds.Builder();
            LatLng me = new LatLng(latitude, longitude);
            builder.include(me);
            aMap.animateCamera(CameraUpdateFactory.newLatLngBounds(builder.build(), 0));
        }
    }

    @Override
    public void onGeocodeSearched(GeocodeResult geocodeResult, int i) {
//        Log.d(TAG, "onRegeocodeSearched: getAdCode="+geocodeResult.getGeocodeAddressList().get(0).getFormatAddress());
    }

    @Override
    public void onClick(View v) {
        Intent it = null;
        if (v == send) {
            action = "screenshot";
            aMap.getMapScreenShot(this);
        } else if (v == backward) {
            action = "justLook";
            this.finish();
        }
    }


    @Override
    public void onMapScreenShot(Bitmap bitmap) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        if (null == bitmap) {
            return;
        }
        bitmap = BitmapUtils.cropBitmap(bitmap, 445, 175, true);
        String picPath;
        String picName = "location_" + sdf.format(new Date()) + ".png";
        try {
            String file = FileUtils.getDiskCacheDir(this);
            File appCacheDir = new File(file, "");
            picPath = appCacheDir.getPath() + "/" + picName;
            FileOutputStream fos = new FileOutputStream(
//                    Environment.getExternalStorageDirectory() + "/test_"
//                            + sdf.format(new Date()) + ".png");
                    picPath);
            Log.d(TAG, "onMapScreenShot: " + picPath);
            boolean b = bitmap.compress(Bitmap.CompressFormat.PNG, 100, fos);
            try {
                fos.flush();
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                fos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            StringBuffer buffer = new StringBuffer();
            if (b) {
                buffer.append("截屏成功 ");
                Log.d(TAG, "onMapScreenShot: 截屏成功");
            } else {
                buffer.append("截屏失败 ");
                Log.d(TAG, "onMapScreenShot: 截屏失败");
            }
            Toast.makeText(this, buffer.toString(), Toast.LENGTH_SHORT).show();
            String position = editPosition.getText().toString().trim();
            Intent intent = new Intent();
            intent.putExtra("picPath", picPath);
            intent.putExtra("picName", picName);
            intent.putExtra("position", position);
            intent.putExtra("latitude", latitude);
            intent.putExtra("longitude", longitude);
            this.setResult(REQUEST_POSITION, intent);
//            startActivity(intent);
            finish();

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mMapView.onDestroy();
    }


    @Override
    public void beforeTextChanged(CharSequence s, int start, int count, int after) {

    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {

    }

    @Override
    public void afterTextChanged(Editable s) {
        if (s.toString().trim().equals("")) {
            Log.d(TAG, "afterTextChanged: 空");
            send.setClickable(false);
            send.setBackgroundResource(R.drawable.button_circle_edge_cccccc);
        } else {
            Log.d(TAG, "afterTextChanged: 不为空");
            send.setBackgroundResource(R.drawable.button_circle_edge_2ba245);
            send.setClickable(true);
        }
    }


    @Override
    public View getInfoWindow(Marker marker) {

        return null;
    }

    @Override
    public View getInfoContents(Marker marker) {
        return null;
    }

    @Override
    public void onMapClick(LatLng latLng) {

    }

    @Override
    public boolean onMarkerClick(Marker marker) {

        return true;
    }

    @Override
    public void showErrorMessage(String error) {

    }

    @Override
    public void createFunctions() {

    }

    @Override
    public void initViews() {
        binding = GetPositionBinding.inflate(getLayoutInflater());
        View rootView = binding.getRoot();
        setContentView(rootView);

        editPosition = binding.editPosition;
        send = binding.send;
        if (!justShowPosition) {
            editPosition.addTextChangedListener(this);
            send.setOnClickListener(this);
        } else {
            linearLayout = binding.linear;
            linearLayout.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                    0, 3));
            editPosition.setFocusable(false);
            editPosition.setText(address);
            editPosition.setClickable(false);
            editPosition.setLines(2);
            send.setVisibility(View.GONE);
        }
        backward = binding.backward;
        backward.setOnClickListener(this);
    }
}