package quake.quake_android_native_methods.ui;

import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Vibrator;
import android.util.Log;
import android.view.View;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;


import cn.bingoogolapple.qrcode.core.QRCodeView;
import cn.bingoogolapple.qrcode.zxing.ZXingView;
import quake.quake_android_native_methods.baseMvp.BasePresenter;
import quake.quake_android_native_methods.databinding.ActivityScanningBinding;
import quake.quake_android_native_methods.ui.ZXing.CodeUtils;
import quake.quake_android_native_methods.ui.base.SlidingBackActivity;
import quake.quake_android_native_methods.utils.Constant;
import quake.quake_android_native_methods.utils.ImageUtils;

public class Scanning extends SlidingBackActivity implements QRCodeView.Delegate, View.OnClickListener {
    private ImageView backward;
    private ZXingView mZXingView;
    private boolean isScan;
    private ImageView gallery;
    private ImageView flashlight;
    private TextView openFlashlight;



    private boolean isStartFlash = false;
    private int REQUEST_IMAGE = 1;
    private boolean animateStart = false;

    private ActivityScanningBinding binding;
    @RequiresApi(api = Build.VERSION_CODES.N)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        initViews();

    }

    @Override
    protected BasePresenter createPresenter() {
        return null;
    }

    @RequiresApi(api = Build.VERSION_CODES.M)
    @Override
    protected void onResume() {
        super.onResume();
        //打开后置摄像头开始预览，但是并未开始识别
        mZXingView.startCamera();

//        //显示扫描框，并且延迟0.1秒后开始识别
        mZXingView.startSpotAndShowRect();
    }

    @Override        //停止
    protected void onStop() {
        super.onStop();
        //关闭摄像头预览，并且隐藏扫描框
        mZXingView.stopCamera();
    }

    @Override        //销毁
    protected void onDestroy() {
        super.onDestroy();
        mZXingView.closeFlashlight();
        mZXingView.onDestroy();
//        Toast.makeText(Scanning.this, 11 + "", Toast.LENGTH_SHORT).show();
    }
    @Override
    public void onClick(View v) {
        Intent intent;
        if (v == backward) {
            finish();
//                            flash.switchFlash();
        } else if (v == gallery) {
            intent = new Intent();
            intent.setAction(Intent.ACTION_PICK);
            intent.setType("image/*");
            startActivityForResult(intent, REQUEST_IMAGE);
        } else if (v ==flashlight) {
            if (!isStartFlash) {
                mZXingView.openFlashlight();
                isStartFlash = true;
            } else if (isStartFlash) {
                mZXingView.closeFlashlight();
                isStartFlash = false;
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == REQUEST_IMAGE) {
            if (data != null) {
                Uri uri = data.getData();
                try {
                    CodeUtils.analyzeBitmap(ImageUtils.getImageAbsolutePath(this, uri), new CodeUtils.AnalyzeCallback() {
                        @Override
                        public void onAnalyzeSuccess(Bitmap mBitmap, String result) {
//                            Toast.makeText(Scaning.this, "解析结果:" + result, Toast.LENGTH_LONG).show();
                            vibrate();//兼容API23
                            Intent intent = new Intent();
                            intent.putExtra("result",result);
                            intent.putExtra("code",1500);
                            setResult(Constant.SCAN_QR_CODE,intent);
                            finish();
                        }

                        @Override
                        public void onAnalyzeFailed() {
                            Toast.makeText(Scanning.this, "解析二维码失败", Toast.LENGTH_LONG).show();
                            Intent intent = new Intent();
                            intent.putExtra("result","解析二维码失败");
                            intent.putExtra("code",1400);
                            setResult(Constant.SCAN_QR_CODE,intent);
                            finish();
                        }
                    });
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }


    private void vibrate() {
        Vibrator vibrator = (Vibrator) getSystemService(VIBRATOR_SERVICE);
        vibrator.vibrate(200);
    }

    // 处理扫描结果
    @Override
    //@param result 摄像头扫码时只要回调了该方法 result 就一定有值，不会为 null。解析本地图片或 Bitmap 时 result 可能为 null
    public void onScanQRCodeSuccess(String result) {
        mZXingView.startSpot();
        if (result != null && !isScan) {
            isScan = true;
            vibrate();//兼容API23
            Intent intent = new Intent();
            intent.putExtra("result",result);
            setResult(Constant.SCAN_QR_CODE,intent);
            finish();
        }

        Log.d("scan", "key:" + result);
//        Toast.makeText(Scanning.this, result, Toast.LENGTH_SHORT).show();
    }

    @Override//摄像头环境亮度发生变化
    // @param isDark 是否变暗
    public void onCameraAmbientBrightnessChanged(boolean isDark) {
        Animation lightOff = new AlphaAnimation(1.0f,0.0f);
        lightOff.setDuration(500);
        lightOff.setRepeatCount(-1);
        lightOff.setRepeatMode(Animation.REVERSE);
        if (isDark&&!animateStart) {
            animateStart = true;
            openFlashlight.setVisibility(View.VISIBLE);
                Log.d("flashlight","lightOff.hasEnded()"+lightOff.hasEnded());
                Log.d("flashlight","lightOff.hasStarted()"+lightOff.hasStarted());
                if(!lightOff.hasStarted()||lightOff.hasEnded()){
                    flashlight.startAnimation(lightOff);
                }
        }
        else if(!isDark){
            animateStart = false;
            openFlashlight.setVisibility(View.INVISIBLE);
            flashlight.clearAnimation();
        }
    }

    @Override//处理打开相机出错
    public void onScanQRCodeOpenCameraError() {
//        mZXingView.stopCamera();
    }

    @Override
    public void showErrorMessage(String error) {

    }

    @Override
    public void createFunctions() {

    }

    @Override
    public void initViews() {
        binding = ActivityScanningBinding.inflate(getLayoutInflater());
        View rootView = binding.getRoot();
        setContentView(rootView);
        mZXingView = binding.zxingview;
        mZXingView.setDelegate(this);

        backward = binding.backward;
        gallery = binding.gallery;
        flashlight = binding.flashlight;
        openFlashlight = binding.openFlashlight;
        backward.setOnClickListener(this);
        gallery.setOnClickListener(this);
        flashlight.setOnClickListener(this);
    }
}