/**
 * Copyright (C), 2021-2021, 夸克科技有限公司
 * FileName: UriUtils
 * Author: Administrator
 * Date: 2021/10/21 17:47
 * Description: uri工具类
 * History:
 * <author> <time> <version> <desc>
 * 作者姓名 修改时间 版本号 描述
 */
package quake.quake_android_native_methods.utils;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.media.MediaMetadataRetriever;
import android.net.Uri;
import android.os.Environment;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.util.Log;



import java.io.File;

/**
 * @ClassName: UriUtils
 * @Description: java类作用描述
uri工具类
 * @Author: Administrator
 * @Date: 2021/10/21 17:47
 */
public class UriUtils {
    private static final String TAG = "UriUtils";

    /**
     * 获取视频的第一帧作为封面
     * @param context
     * @param uri
     * @return
     */
    public  static Bitmap getVideoFirstFrameByUri(Context context, Uri uri){
        //获取视频的第一帧作为封面
        Bitmap bitmap;
        MediaMetadataRetriever media = null;
        try {
            media = new MediaMetadataRetriever();

            String videoAbsolutePath = ImageUtils.getImageAbsolutePath(context, uri);           // 通过Uri获取绝对路径
            Log.d(TAG, "onActivityResult: 绝对路径=" + videoAbsolutePath);
//                    FileInputStream inputStream = new FileInputStream(new File(videoPath).getAbsolutePath());
//                    if (Build.VERSION.SDK_INT >= 14) {
//                        media.setDataSource(inputStream.getFD(), new HashMap<String, String>());
//                    } else {
            media.setDataSource(videoAbsolutePath);
            bitmap = media.getFrameAtTime();      // 视频的第一帧图片
        } finally {
            media.release();
        }
        return bitmap;
    }


    public static String getPathFromUri(Context context, Uri uri) {
        String path = null;
        if (DocumentsContract.isDocumentUri(context, uri)) {
            //如果是document类型的Uri，通过document id处理，内部会调用Uri.decode(docId)进行解码
            String docId = DocumentsContract.getDocumentId(uri);
            //primary:Azbtrace.txt
            //video:A1283522
            String[] splits = docId.split(":");
            String type = null, id = null;
            if(splits.length == 2) {
                type = splits[0];
                id = splits[1];
            }
            switch (uri.getAuthority()) {
                case "com.android.externalstorage.documents":
                    if("primary".equals(type)) {
                        path = Environment.getExternalStorageDirectory() + File.separator + id;
                    }
                    break;
                case "com.android.providers.downloads.documents":
                    if("raw".equals(type)) {
                        path = id;
                    } else {
                        Uri contentUri = ContentUris.withAppendedId(Uri.parse("content://downloads/public_downloads"), Long.valueOf(docId));
                        path = getMediaPathFromUri(context, contentUri, null, null);
                    }
                    break;
                case "com.android.providers.media.documents":
                    Uri externalUri = null;
                    switch (type) {
                        case "image":
                            externalUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
                            break;
                        case "video":
                            externalUri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI;
                            break;
                        case "audio":
                            externalUri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
                            break;
                        case "document":
                            externalUri = MediaStore.Files.getContentUri("external");
                            break;
                    }
                    if(externalUri != null) {
                        String selection = "_id=?";
                        String[] selectionArgs = new String[]{ id };
                        path = getMediaPathFromUri(context, externalUri, selection, selectionArgs);
                    }
                    break;
            }
        } else if (ContentResolver.SCHEME_CONTENT.equalsIgnoreCase(uri.getScheme())) {
            path = getMediaPathFromUri(context, uri, null, null);
        } else if (ContentResolver.SCHEME_FILE.equalsIgnoreCase(uri.getScheme())) {
            //如果是file类型的Uri(uri.fromFile)，直接获取图片路径即可
            path = uri.getPath();
        }
        //确保如果返回路径，则路径合法
        return path == null ? null : (new File(path).exists() ? path : null);
    }

    private static String getMediaPathFromUri(Context context, Uri uri, String selection, String[] selectionArgs) {
        String path;
        String authority = uri.getAuthority();
        path = uri.getPath();
        String sdPath = Environment.getExternalStorageDirectory().getAbsolutePath();
        if(!path.startsWith(sdPath)) {
            int sepIndex = path.indexOf(File.separator, 1);
            if(sepIndex == -1) path = null;
            else {
                path = sdPath + path.substring(sepIndex);
            }
        }

        if(path == null || !new File(path).exists()) {
            ContentResolver resolver = context.getContentResolver();
            String[] projection = new String[]{ MediaStore.MediaColumns.DATA };
            Cursor cursor = resolver.query(uri, projection, selection, selectionArgs, null);
            if (cursor != null) {
                if (cursor.moveToFirst()) {
                    try {
                        int index = cursor.getColumnIndexOrThrow(projection[0]);
                        if (index != -1) path = cursor.getString(index);
                        Log.i(TAG, "getMediaPathFromUri query " + path);
                    } catch (IllegalArgumentException e) {
                        e.printStackTrace();
                        path = null;
                    } finally {
                        cursor.close();
                    }
                }
            }
        }
        return path;
    }
    //创建照片存储uri
    public static Uri createImageUri(Context context) {
        String name = "takePhoto" + System.currentTimeMillis();
        String path = Environment.getExternalStorageDirectory().getPath()+"/Quake/Photograph/";
        File file = new File(path);
        //创建目录
        if (!file.exists()) {
            file.mkdirs();
        }
        //获取默认存储地址，而且指定文件夹
        //File.separator 是默认分隔符 \
        String picPath = path + name + ".jpg";
        ContentValues contentValues = new ContentValues(1);
        contentValues.put(MediaStore.Images.Media.DATA, picPath);
        Uri uri = context.getContentResolver().insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, contentValues);
        SharedPreferences sharedPreferences = context.getSharedPreferences("photoPath", Activity.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString("path", picPath);
        editor.commit();
        return uri;
    }
}
