package com.fashare.hover_view;

import android.content.Context;


import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.core.view.ViewCompat;
import androidx.customview.widget.ViewDragHelper;
import androidx.drawerlayout.widget.DrawerLayout;

/**
 * User: fashare(153614131@qq.com)
 * Date: 2016-12-12
 * Time: 21:16
 *
 * {@link HoverView} 的容器,
 * 类似于{@link DrawerLayout}
 */
public class HoverViewContainer extends FrameLayout implements ViewStateManager {
    private final String TAG = this.getClass().getSimpleName()+"容器";
    private Context mContext;

    private ViewDragHelper mBottomDragger;
    private static final float TOUCH_SLOP_SENSITIVITY = 1.f;
    private DragCallback mBottomCallback = new DragCallback();

    private HoverView mBottomView;
    static HoverView EMPTY_VIEW;

    public ViewState getmViewState() {
        return mViewState;
    }

    public void setmViewState(ViewState mViewState) {
        this.mViewState = mViewState;
    }

    private class DragCallback extends ViewDragHelper.Callback{
        // 仅捕获 mBottomView
        @Override
        public boolean tryCaptureView(View child, int pointerId) {
            return child == mBottomView;
        }

        // 控制边界, 防止 mBottomView 的头部超出边界
        @Override
        public int clampViewPositionVertical(View child, int top, int dy) {

            if(child == mBottomView){
                int newTop = top;
                newTop = Math.max(newTop, ViewState.FILL.getTop(mBottomView));
                return newTop;
            }
            return top;
        }

        @Override
        public void onViewPositionChanged(@NonNull View changedView, int left, int top, int dx, int dy) {
            super.onViewPositionChanged(changedView, left, top, dx, dy);
        }

        // 手指释放的时候回调
        @Override
        public void onViewReleased(View releasedChild, float xvel, float yvel) {

            if (releasedChild == mBottomView) {
                int curTop = releasedChild.getTop();
                setClosestStateIfBetween(ViewState.FILL, ViewState.HOVER, curTop);
                setClosestStateIfBetween(ViewState.HOVER, ViewState.CLOSE, curTop);
            }
            Log.d(TAG, "getState: 释放");
            getState(mViewState,true);
        }

        //设置最接近的状态，如果在两者之间
        private void setClosestStateIfBetween(ViewState beginState, ViewState endState, int curTop){
            Log.d(TAG, "setClosestStateIfBetween: 是你调用了两次吧");
            int beginTop = getTopOfState(beginState),
                    endTop = getTopOfState(endState);
            Log.d(TAG, "setClosestStateIfBetween: 开始="+beginTop);
            Log.d(TAG, "setClosestStateIfBetween: 结束="+endTop);
            Log.d(TAG, "setClosestStateIfBetween: 变量="+curTop);
            if(curTop >= beginTop && curTop <= endTop)
                changeState(curTop < (beginTop + endTop)/2? beginState: endState);

            Log.d(TAG, "setClosestStateIfBetween: mViewState="+mViewState);
        }
    }

    int getTopOfState(ViewState viewState){

        return viewState.getTop(mBottomView);
    }

    public HoverViewContainer(Context context) {
        super(context);
        init(context);
    }

    public HoverViewContainer(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context);
    }

    public HoverViewContainer(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context);
    }

    private void init(Context context) {
        mContext = context;
        mBottomDragger = ViewDragHelper.create(this, TOUCH_SLOP_SENSITIVITY, mBottomCallback);
        mBottomDragger.setEdgeTrackingEnabled(ViewDragHelper.EDGE_BOTTOM);

        EMPTY_VIEW = new HoverView(context);
        EMPTY_VIEW.setLayoutParams(new FrameLayout.LayoutParams(0, 0));
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        super.onLayout(changed, left, top, right, bottom);
        Log.d(TAG, "onLayout: mViewState="+mViewState);
        if(mViewState == ViewState.HOVER){
            changeState(ViewState.HOVER, false);
        }
        else if(mViewState == ViewState.FILL){
            changeState(ViewState.FILL, false);
        }else {
            changeState(ViewState.CLOSE, false);
        }
        Log.d(TAG, "onLayout: 谁调用的这破玩意儿");
    }

    @Override
    protected void onFinishInflate() {
        super.onFinishInflate();

        mBottomView = findHoverView();
        if(mBottomView == EMPTY_VIEW)
            this.addView(EMPTY_VIEW);
    }

    private HoverView findHoverView() {
        for(int i=0; i<getChildCount(); i++)
            if(getChildAt(i) instanceof HoverView)
                return (HoverView)getChildAt(i);

        return EMPTY_VIEW;
    }

    @Override
    public boolean onInterceptTouchEvent(MotionEvent ev) {
        return mBottomDragger.shouldInterceptTouchEvent(ev);
    }

    @Override
    public boolean onTouchEvent(MotionEvent ev) {
        mBottomDragger.processTouchEvent(ev);
        return true;
    }

    // ------ 滑动部分 begin ------
    private void smoothScrollTo(int finalTop){
        mBottomDragger.smoothSlideViewTo(mBottomView, 0, finalTop);
        ViewCompat.postInvalidateOnAnimation(this);

    }

    // smoothScrollTo() 中用到 mScroller,
    // 不要忘了配合 computeScroll()
    @Override
    public void computeScroll() {
        if(mBottomDragger.continueSettling(true))
            ViewCompat.postInvalidateOnAnimation(this);

    }

    private void scrollTo(int finalTop){
        ViewCompat.offsetTopAndBottom(mBottomView, finalTop - getTop());
        ViewCompat.postInvalidateOnAnimation(this);

    }
    // ------ 滑动部分 end ------

    // ------ 对外接口 ViewStateManager: begin ------
    private ViewState mViewState = ViewState.CLOSE;

    @Override
    public void changeState(ViewState viewState){

     changeState(viewState, true);

    }

    @Override
    public void changeState(ViewState viewState, boolean isSmoothScroll) {
        mViewState = viewState;
        if(isSmoothScroll)
            smoothScrollTo(getTopOfState(viewState));
        else
            scrollTo(getTopOfState(viewState));
        Log.d(TAG, "changeState: 你调了几次");
        Log.d(TAG, "changeState: mViewState="+mViewState);
        Log.d(TAG, "changeState: 改变状态="+viewState);
    }

    @Override
    public ViewState getState() {
        Log.d(TAG, "getState: 当前状态="+mViewState);
        return mViewState;
    }
    // ------ 对外接口 ViewStateManager: end ------
    private RecordingWindowListener mWindowListener;
    public void setRecordingWindowListener(RecordingWindowListener listener){
        mWindowListener = listener;
    }

    public interface RecordingWindowListener {
        void windowState(ViewState state,boolean isSmoothScroll);
    }

    public void getState(ViewState state, boolean isSmoothScroll){
        Log.d(TAG, "doSomething: "+state);
        if(mWindowListener!=null){
            mWindowListener.windowState(state,isSmoothScroll);
        }
    }
}
