import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:quake_android_native_methods/quake_android_native_methods.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    // We also handle the message potentially returning null.
    try {
      platformVersion = await QuakeAndroidNativeMethods.platformVersion ??
          'Unknown platform version';
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: Column(
            children: <Widget>[
              Text('Running on: $_platformVersion\n'),
              Text('返回值: $_platformVersion\n'),
              RaisedButton(
                child: const Text(
                  '打开相册',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                      await QuakeAndroidNativeMethods.chooseImgFromGallery();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '拍照',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                      await QuakeAndroidNativeMethods.photograph();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '打开视频',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                      await QuakeAndroidNativeMethods.chooseVideoFromSelector();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '录像',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message = await QuakeAndroidNativeMethods.videoRecording();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '打开文件',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                      await QuakeAndroidNativeMethods.chooseFileFromSelector();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '查看文件',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                  await QuakeAndroidNativeMethods.browseFile('https://quake-hr-1253674864.cos.ap-guangzhou.myqcloud.com/dailyFiles/13/1103/2021/10/29/新建文本文档.txt');
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '录音',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                      await QuakeAndroidNativeMethods.soundRecording();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '发送定位',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                      await QuakeAndroidNativeMethods.sendLocation();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
              RaisedButton(
                child: const Text(
                  '扫描二维码',
                  style: TextStyle(
                    fontSize: 28.0,
                  ),
                ),
                onPressed: () async {
                  String? message =
                  await QuakeAndroidNativeMethods.scanQRCode();
                  // ignore: avoid_print
                  print("Dart层：" + message!);
                },
              ),
            ],
          ),
        ),
      ),
    );
  }
}
